/* scherzo - Music notation training
 *
 * Copyright © 2010 Carl Worth
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/ .
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "score.h"

#define unused(foo) foo __attribute__((unused))

typedef struct scherzo
{
    score_t *score;
    int staff_height;
} scherzo_t;

static int
on_delete_event_quit (unused (GtkWidget *widget),
		      unused (GdkEvent *event),
		      unused (gpointer user_data))
{
    gtk_main_quit ();

    /* Returning FALSE allows the default handler for delete-event
     * to proceed to cleanup the widget. */
    return FALSE;
}

static int
on_expose_event_draw (GtkWidget *widget,
		      unused (GdkEventExpose *expose),
		      void * user_data)
{
    scherzo_t *scherzo = user_data;
    score_t *score = scherzo->score;
    cairo_t *cr;
    GtkAllocation allocation;
    static const int pad = 10;
    int widget_width;

    gtk_widget_get_allocation (widget, &allocation);
    widget_width = allocation.width;

    cr = gdk_cairo_create (widget->window);

    /* White background */
    cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
    cairo_paint (cr);

    /* Add some padding on the sides and top */
    cairo_translate (cr, pad, (int) scherzo->staff_height / 2);
    score_set_staff_height (score, scherzo->staff_height);
    score_set_width (score, widget_width - 2 * pad);

    score_draw (score, cr);
 
    return TRUE;
}

static int
on_key_press_event (GtkWidget *widget,
		    GdkEventKey *key,
		    void *user_data)
{
    scherzo_t *scherzo = user_data;

    switch (key->keyval) {
    case GDK_KEY_plus:
    case GDK_KEY_KP_Add:
    case GDK_KEY_equal:
    case GDK_KEY_KP_Equal:
	scherzo->staff_height += 4;
	gtk_widget_queue_draw (widget);
	return TRUE;
	break;
    case GDK_KEY_minus:
    case GDK_KEY_KP_Subtract:
	scherzo->staff_height -= 4;
	gtk_widget_queue_draw (widget);
	return TRUE;
	break;
    case GDK_KEY_q:
    case GDK_KEY_Q:
    case GDK_KEY_Escape:
	gtk_main_quit ();
	return FALSE;
    }

    /* Allow the event to propagate to other handlers. */
    return FALSE;
}

int
main (int argc, char *argv[])
{
    GtkWidget *window;
    GtkWidget *drawing_area;
    scherzo_t scherzo;

    gtk_init (&argc, &argv);

    scherzo.score = score_create (NULL);
    scherzo.staff_height = 48;
    score_set_staff_height (scherzo.score, scherzo.staff_height);

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

    gtk_window_set_default_size (GTK_WINDOW (window), 600, 400);

    g_signal_connect (window, "delete-event",
		      G_CALLBACK (on_delete_event_quit), NULL);

    drawing_area = gtk_drawing_area_new ();

    gtk_container_add (GTK_CONTAINER (window), drawing_area);

    g_signal_connect (drawing_area, "expose-event",  
		      G_CALLBACK (on_expose_event_draw),
		      &scherzo);

    g_signal_connect (window, "key-press-event",  
		      G_CALLBACK (on_key_press_event),
		      &scherzo);
    
    gtk_widget_show_all (window);
    
    gtk_main ();

    return 0;
}
