/* scherzo - Music notation training
 *
 * 	score - Utilities for drawing (simple) musical scores
 *
 * Copyright © 2010 Carl Worth
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/ .
 */

#ifndef SCORE_H
#define SCORE_H

#include <talloc.h>
#include <cairo.h>

typedef struct score score_t;

/* Allocate a new, empty score object, (with optional ctx as talloc
 * owner). If ctx is NULL, the caller should call talloc_free on the
 * score_t* when done with it. Otherwise, the object will be freed
 * when ctx is freed. */
score_t *
score_create (void *ctx);

/* Set an (approximate) staff height. The actual staff height may
 * differ due to rounding to achieve evenly spaced, sharply rendered
 * lines. the actual staff height is returned. */
int
score_set_staff_height (score_t *score, int height);

/* Set the total width available for drawing the score. */
void
score_set_width (score_t *score, int width);

/* Draw the given score_t onto the given cairo_t.
 *
 * The caller can call cairo_translate before calling score_draw to
 * position the result as desired, (and can call cairo_clip to clip it
 * if desired). */
void
score_draw (score_t *score, cairo_t *cr);

#endif
